@@ -248,7 +248,10 @@ ppc64_vmalloc_init(void)
 
 			if (info->kernel_version >= KERNEL_VERSION(4, 12, 0)) {
 				info->l2_index_size = PMD_INDEX_SIZE_L4_64K_4_12;
-				info->l3_index_size = PUD_INDEX_SIZE_L4_64K_4_12;
+				if (info->kernel_version >= KERNEL_VERSION(4, 17, 0))
+					info->l3_index_size = PUD_INDEX_SIZE_L4_64K_4_17;
+				else
+					info->l3_index_size = PUD_INDEX_SIZE_L4_64K_4_12;
 				info->l4_index_size = PGD_INDEX_SIZE_L4_64K_4_12;
 			} else {
 				info->l2_index_size = PMD_INDEX_SIZE_L4_64K_4_6;
@@ -478,6 +481,16 @@ set_ppc64_max_physmem_bits(void)
 		|| (array_len == (NR_MEM_SECTIONS() / _SECTIONS_PER_ROOT())))
 		return TRUE;
 
+	info->max_physmem_bits  = _MAX_PHYSMEM_BITS_4_19;
+	if ((array_len == (NR_MEM_SECTIONS() / _SECTIONS_PER_ROOT_EXTREME()))
+		|| (array_len == (NR_MEM_SECTIONS() / _SECTIONS_PER_ROOT())))
+		return TRUE;
+
+	info->max_physmem_bits  = _MAX_PHYSMEM_BITS_4_20;
+	if ((array_len == (NR_MEM_SECTIONS() / _SECTIONS_PER_ROOT_EXTREME()))
+		|| (array_len == (NR_MEM_SECTIONS() / _SECTIONS_PER_ROOT())))
+		return TRUE;
+
 	return FALSE;
 }
 
@@ -644,7 +657,7 @@ int arch_crashkernel_mem_size_ppc64()
 		return FALSE;
 	}
 	fpb = fopen(f_crashbase, "r");
-	if (!fp) {
+	if (!fpb) {
 		ERRMSG("Cannot open %s\n", f_crashbase);
 		fclose(fp);
 		return FALSE;
